<?php

// --------------------------------------------------------------------
// GLOBALS
// --------------------------------------------------------------------

define('ROOT_REL', dirname($_SERVER['PHP_SELF']));
define('HOST_URL', 'http://' . $_SERVER['HTTP_HOST']);
define('SITE_URL', HOST_URL . dirname($_SERVER['PHP_SELF']));

// --------------------------------------------------------------------
// Array functions
// --------------------------------------------------------------------

//! A formatted print_r.
function aprint($arr, $ret=false)
{
	$text= "\n<pre>" . print_r($arr, true) . "</pre>\n";
	if($ret)
		return $text;
	else
		echo $text;
}

function aget($key, $array, $default, $format="%s")
{
	if(!empty($array[$key]))
		return str_replace("%s", $array[$key], $format);
	else
		return $default;
}

//! Array variant for pack(), with recursion.
function apack($a)
{
	$s= "";
	$result= array();
	$map= array('u8'  => 'C', 'u16' => 'n', 'u32' => 'N');
	
	foreach($a as $item)
	{
		if(!is_array($item[0]))
		{
			$format= (isset($map[$item[0]]) ? $map[$item[0]] : $item[0]);
			$s .= pack($format, $item[1]);
		}
		else 
			$s .= apack($item);
	}
	return $s;
}

//! Call a procedure from a list of procedures.
/*!
	@param procs	Procedure list.
	@param key		Key (int or string) to the proc. If not found, the 'default' 
					is run.
	@param opts		Options to be passed to the procedure.
	@return			Return value of the called procedure, or false.
*/
function acall($procs, $key, $opts= array())
{
	if(isset($procs[$key]))
		return call_user_func_array($procs[$key], $opts);
	else if($procs['default'])
		return call_user_func_array($procs['default'], $opts);
	else
		return false;
}

// --------------------------------------------------------------------
// Base functions
// --------------------------------------------------------------------

//! Get the value of \a $item if it exists, or get the default value.
/*!	@note	Note the use of 'empty'. SimpleXMLElements are quirky when using 
			anything else.
*/
function get($item, $default, $format="%s")
{
	if(isset($item) && !empty($item))
		return str_replace("%s", $item, $format);
	else
		return $default;
}

//! Check if a string is blank or non-existent.
function isBlank($str)
{
	return !isset($str) || (strlen(trim($str)) == 0);
}

//! Fixes site-relative URLs to absolute URLS.
/*! Relative links screw up pretty permalinks something fierce. However, absolute 
	links can't be used either because it makes moving sites impossible, and 
	root-relative won't work because I'm not at root-level. 
	A 'fix' is to create a symbol for 'site-relative' links: links that are 
	relative to the site's root, not the server's root. This symbol will be 
	"~/" (compare "./" and "/" and the unix 'home'). This function will convert 
	these to root-relative links.
	@return	Converted content.
*/
function urlFix($content, $bQuoted= true)
{
	$root= (ROOT_REL=="/" ? "" : ROOT_REL);
	if($bQuoted)
		return preg_replace('#"~\\\?/(.*?")#', "\"$root/$1", $content);
	else
		return str_replace("~/", "$root/", $content);
}

//! Indent each line of a multi-line string with \a $prefix.
/*!
	@param $content		Text to indent.
	@param $prefix		String to indent the lines of \a $content with ('\t').
	@param $count		Number of times to indent with \a $prefix (1).
*/
function indent($content, $prefix= "\t", $count=1)
{
	$prefix= str_repeat($prefix, $count);
	return $prefix . str_replace("\n", "\n$prefix", $content);
}

//! Simple HTTP_POST method via cURL.
function curler($url, $vars= array())
{
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $vars);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_TIMEOUT, 10);
	$result = curl_exec($ch);
	curl_close($ch);
	
	return $result;
}

// -----------------------------------------------------------------------------
// SINGLETONS
// -----------------------------------------------------------------------------

//! Legacy debug interface.
function debugInit($bDebug)
{
	debug('init', $bDebug);
}

//! Immediate logging / printing of debug items.
/*!	@note	The functionality of this can probably be expanded a little more.
*/
function debug($cmd, $msg= "")
{
	static $dbg= null;
	if($dbg == null && $cmd != 'init')
		debug('init', false);
		
	switch($cmd)
	{
	case 'init':
		$dbg= array(
			'dbg' => !empty($msg)
		);
		if($dbg['dbg'])
		{
			//echo "<div>DEBUG MODE! " . SITE_URL . "</div>";
			ini_set('display_errors', 1); 
			error_reporting(E_ALL);
		}
		else
			error_reporting(E_ERROR);
		break;
	
	case 'dump':	if($dbg['dbg']) echo($msg);		break;
	case 'a':		if($dbg['dbg']) aprint($msg);	break;
	case 'echo':
	default:		if($dbg['dbg']) echo($msg);		break;
	}
}

//! Buffered logging.
function logger($cmd, $msg="", $return=false)
{
	static $log= null;
	if($log == null && $cmd != 'init')
		logger('init');

	$types= array('error', 'status', 'warning');
	switch($cmd)
	{
	case 'init':
	case 'clear':
		$log= array(
			'stats' => array(),
			'msgs' => array()
		);
		foreach($types as $type)
			$log['stats'][$type]= 0;
			
		return true;

	case 'stats':
		return $log['stats'];
	
	case 'get':
		$content  = "<report>\n";
		$stats= "";
		$msgs= "";

		switch($msg)
		{
		case 'html':
			//foreach($log['stats'] as $k => $v)
			//	$stats .= "\t\t<div class=\"{$k}_count\">$v</div>\n";
			foreach($log['msgs'] as $v)
				$msgs .= "\t\t<div class=\"{$v['type']}\">{$v['msg']}</div>\n";
			$content= <<<DOC
<div id="report">
	<div class="stats">\n$stats\n\t</div>
	<div class="messages">\n$msgs\n\t</div>
</div>\n
DOC;
			break;

		default:
			foreach($log['stats'] as $k => $v)
				$stats .= "\t\t<{$k}_count>$v</{$k}_count>\n";
			foreach($log['msgs'] as $v)
				$msgs .= "\t\t<{$v['type']}>{$v['msg']}</{$v['type']}>\n";
			$content= <<<DOC
<report>
	<stats>\n$stats\n\t</stats>
	<messages>\n$msgs\n\t</messages>
</report>\n
DOC;
		}

		return $content;
		
	case 'error':
	case 'status':
	case 'warning':
		$log['stats'][$cmd]++;
		array_push($log['msgs'], array('type' => $cmd, 'msg' => $msg));
		break;
	}

	return $return;
}

// --------------------------------------------------------------------
// Math functions
// --------------------------------------------------------------------

//! Clamp x between \a lb and \a ub.
/*!
	@param x	value to clamp.
	@param lb	Lower bound (inclusive)
	@param ub	Upper bound (exclusive)
	@return		x in [lb, ub>
*/
function clamp($x, $lb, $ub)
{
	return ($x >= $ub ? $ub-1 : ($x < $lb ? $lb : $x));
}

// --------------------------------------------------------------------
// HTML functions
// --------------------------------------------------------------------

//! HTML starting boilerplate.
/*!
	@param $opts		Various options:
	- 'css' (array)		Array of CSS urls.
	- 'favicon' (string)	fav-icon url [ "./favicon.ico" ].
	- 'js' (array)		Array of JS urls.
	- 'meta' (array)	Array with meta name-content items.
	- 'title' (string)	Browser title.
	- 'custom' (string)	Custom entries; added directly to header.
*/
function htmlBegin($opts)
{
	$opts= array_merge(array(
		'doctype' =>	'HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"',
		'charset' =>	"UTF-8",
		'css' => 		array(), 
		'favicon' =>	SITE_URL . "/favicon.ico", 
		'js' => 		array(), 
		'meta' =>		array(), 
		'title' =>		"", 
		'custom' =>		"",
	), $opts);
	
	$charset = "  <meta http-equiv=\"Content-Type\" content=\"text/html;charset={$opts['charset']}\">\n";
	
	$meta= "";
	foreach($opts['meta'] as $k => $v)
		$meta .= "  <meta name=\"$k\" content=\"$v\">\n";

	$css= "";
	foreach($opts['css'] as $v)
		$css .= "  <link rel=\"stylesheet\" type=\"text/css\" href=\"$v\">\n";

	$js= "";
	foreach($opts['js'] as $v)
		$js .= "  <script type=\"text/javascript\" src=\"$v\"></script>\n";
	
	$paths= array('site' => SITE_URL, 'root' => ROOT_REL);
	$content= <<<DOC
<!DOCTYPE {$opts['doctype']}>
<html>
<head>
  <title>{$opts['title']}</title>
$charset
$meta
  <link rel="shortcut icon" href="{$opts['favicon']}">
$css
$js
{$opts['custom']}
<script type="text/javascript">
var SITE_URL= "{$paths['site']}", SITE_ROOT= "{$paths['root']}";
</script>
</head>
<body>\n
DOC;
	echo urlfix($content);
}

//! Create an HTML table from a 2D array.
function htmlTable($mtx)
{
	if(count($mtx) == 0)
		return "";
		
	$keys= array_keys($mtx[0]);

	$header= "  <tr><th>" . implode("</th><th>", $keys) . "</th></tr>\n";
	$content= "";
	foreach($mtx as $a)
	{
		$row= "";
		foreach($keys as $k)
			$row .= "    <td>" . (isset($a[$k]) ? $a[$k] : "") . "</td>\n";
		$content .= "  <tr>\n$row  </tr>\n";
	}
	
	return "<table>$header$content</table>";
}

//! HTML finalizer.
function htmlEnd()
{
	echo <<<DOC
</body>
</html>\n
DOC;
}

?>